;(function (sofa, document, undefined) {

'use strict';
/* global navigator */
/* global document */
/* global sofa */
/**
 * @name DeviceService
 * @namespace sofa.DeviceService
 *
 * @description
 * This is a helper service that gives you methods to check for certain contexts
 * on touch devices etc.. It determines the state for the usage of flexbox as well
 * as things like overflow:scroll support.
 */
sofa.define('sofa.DeviceService', function ($window) {
    var self = {};

    var ua = navigator.userAgent,
        htmlTag,
        isIpadOnIos7,
        uaindex,
        userOS,
        userOSver;

    var MODERN_FLEXBOX_SUPPORT = 'cc-modern-flexbox',
        NO_MODERN_FLEXBOX_SUPPORT = 'cc-no-modern-flexbox',
        IPAD_ON_IOS_7 = 'cc-ipad-ios-7';

    /**
     * @method getHtmlTag
     * @memberof cc.DeviceService
     *
     * @description
     * Returns an HTMLDomObject for HTML.
     *
     * @return {object} HTMLDomObject
     */
    self.getHtmlTag = function () {
        htmlTag = htmlTag || document.getElementsByTagName('html')[0];
        return htmlTag;
    };

    // determine OS
    if (ua.match(/iPad/i) || ua.match(/iPhone/i)) {
        userOS = 'iOS';
        uaindex = ua.indexOf('OS ');
    }
    else if (ua.match(/Android/i)) {
        userOS = 'Android';
        uaindex = ua.indexOf('Android ');
    }
    else {
        userOS = 'unknown';
    }

    // determine version
    if (userOS === 'iOS'  &&  uaindex > -1) {
        userOSver = ua.substr(uaindex + 3, 3).replace('_', '.');
    } else if (userOS === 'Android'  &&  uaindex > -1) {
        userOSver = ua.substr(uaindex + 8, 3);
    } else {
        userOSver = 'unknown';
    }

    // determine iPad + iOS7 (for landscape innerHeight bug, see flagIpadOnIos7() )
    isIpadOnIos7 = ua.match(/iPad/i) && userOSver.substr(0, 1) === '7';

    /**
     * @method isIpadOnIos7
     * @memberof cc.DeviceService
     *
     * @description
     * Returns a boolean indicating whether the device is an iPad running iOS7 or not.
     *
     * @return {boolean}
     */
    self.isIpadOnIos7 = function () {
        return isIpadOnIos7;
    };

    var dimensions = {};

    var updateDimension = function () {
        dimensions.width = $window.innerWidth;
        dimensions.height = $window.innerHeight;
    };

    updateDimension();

    $window.addEventListener('orientationchange', updateDimension, false);

    var versionStartsWith = function (str) {
        var version = self.getOsVersion();
        return version.indexOf(str) === 0;
    };

    /**
     * @method getViewportDimensions
     * @memberof cc.DeviceService
     *
     * @description
     * Returns the height of the viewport
     *
     * @return {int}
     */
    self.getViewportDimensions = function () {
        return dimensions;
    };

    /**
     * @method isInPortraitMode
     * @memberof cc.DeviceService
     *
     * @description
     * Returns a bool indicating whether the decice is held in portrait mode.
     *
     * @return {bool} boolean
     */
    self.isInPortraitMode = function () {
        return dimensions.height > dimensions.width;
    };

    /**
     * @method isLandscapeMode
     * @memberof cc.DeviceService
     *
     * @description
     * Returns a bool indicating whether the decice is held in landscape mode.
     *
     * @return {boolean}
     */
    self.isInLandscapeMode = function () {
        return !self.isInPortraitMode();
    };

    /**
     * @method isTabletSiye
     * @memberof sofa.DeviceService
     *
     * @description
     * Returns true if the current device is in "TabletSize". See SO link for more
     * information (http://stackoverflow.com/questions/6370690/media-queries-how-to-target-desktop-tablet-and-mobile).
     *
     * @return {boolean} Whether the device is in tablet size or not.
     */
    self.isTabletSize = function () {
        return $window.screen.width > 641;
    };

    /**
     * @method isStockAndroidBrowser
     * @memberof sofa.DeviceService
     *
     * @description
     * Checks if browser is stock android browser or not.
     *
     * @return {boolean}
     */
    self.isStockAndroidBrowser = function () {
        return userOS === 'Android' && ua.indexOf('Chrome') < 0;
    };

    /**
     * @method flagOs
     * @memberof sofa.DeviceService
     *
     * @description
     * Flags the current document with an SDK specific class depending on the OS
     * of the device.
     */
    self.flagOs = function () {
        var htmlTag = self.getHtmlTag();
        var version = self.getOsVersion();
        var majorVersion = version.length > 0 ? version[0] : '0';
        htmlTag.className += ' cc-os-' + self.getOs().toLowerCase() + ' cc-osv-' + majorVersion;
    };

    /**
     * @method flagOverflowSupport
     * @memberof sofa.DeviceService
     *
     * @description
     * Flags the current document with an SDK specific class depending on given
     * overflow:scroll support.
     */
    self.flagOverflowSupport = function () {
        var htmlTag = self.getHtmlTag();
        htmlTag.className += self.hasOverflowSupport() ? ' cc-has-overflow-support' : ' cc-has-no-overflow-support';
    };

     /**
      * @method getUserAgent
      * @memberof sofa.DeviceService
      *
      * @description
      *
      * @example
      *
      * @return {string} User agent currently in use
      */
    self.getUserAgent = function () {
        return ua;
    };

    /**
     * @method getOs
     * @memberof sofa.DeviceService
     *
     * @description
     * Returns OS string.
     *
     * @return {string} Name of OS.
     */
    self.getOs = function () {
        return userOS;
    };

    /**
     * @method getOsVersion
     * @memberof sofa.DeviceService
     *
     * @description
     * Returns OS version string.
     *
     * @return {string} Version of OS.
     */
    self.getOsVersion = function () {
        return userOSver;
    };

    /**
     * @method isAndroid2x
     * @memberof sofa.DeviceService
     *
     * @description
     * Returns true if device os is Android and version starts with '2'.
     *
     * @return {bool}
     */
    self.isAndroid2x = function () {
        return self.getOs() === 'Android' && versionStartsWith('2');
    };

    /**
     * @method hasOverflowSupport
     * @memberof sofa.DeviceService
     *
     * @description
     * Checks if the current device is blacklisted as such with no overflow:scroll support
     *
     * @return {boolean}
     */
    self.hasOverflowSupport = function () {
        if (self.getOs() === 'Android') {
            return !versionStartsWith('2');
        } else if (self.getOs() === 'iOS') {
            return  !versionStartsWith('1') &&
                    !versionStartsWith('2') &&
                    !versionStartsWith('3') &&
                    !versionStartsWith('4');
        }
    };

    /**
     * @method hasModernFlexboxSupport
     * @memberof sofa.DeviceService
     *
     * @description
     * Checks if the browser has modern flexbox support or not.
     *
     * @return {boolean}
     */
    self.hasModernFlexboxSupport = function () {

        // Firefox currently has a flexbox bug
        // See http://stackoverflow.com/a/17435156/956278
        if (ua.match(/Firefox/i)) {
            return false;
        }

        var supportedValues = [
            '-webkit-flex',
            '-moz-flex',
            '-o-flex',
            '-ms-flex',
            'flex'
        ];

        var testSpan = document.createElement('span');

        supportedValues.forEach(function (value) {
            testSpan.style.display = value;
        });

        return supportedValues.indexOf(testSpan.style.display) > -1;
    };

    /**
     * @method flagModernFlexboxSupport
     * @memberof sofa.DeviceService
     *
     * @description
     * Flags the document with an SDK specific class for modern flexbox support.
     */
    self.flagModernFlexboxSupport = function () {
        var htmlTag = self.getHtmlTag();
        if (self.hasModernFlexboxSupport()) {
            htmlTag.className += ' ' + MODERN_FLEXBOX_SUPPORT;
        } else {
            htmlTag.className += ' ' + NO_MODERN_FLEXBOX_SUPPORT;
        }
    };

    /**
     * @method flagIpadOnIos7
     * @memberof cc.DeviceService
     *
     * @description
     * Flags the document with an SDK specific class to help getting around a bug in iOS7 on iPad landscape mode.
     * see http://stackoverflow.com/questions/18855642/ios-7-css-html-height-100-692px
     */
    self.flagIpadOnIos7 = function () {
        if (isIpadOnIos7) {
            var htmlTag = self.getHtmlTag();
            htmlTag.className += ' ' + IPAD_ON_IOS_7;
        }
    };

    /**
     * @method setViewportHeightToDeviceHeight
     * @memberof cc.DeviceService
     *
     * @description
     * Sets the height of the html element to the actual height of the device.
     */
    self.setViewportHeightToDeviceHeight = function () {
        self.getHtmlTag().style.height = self.getViewportDimensions().height + 'px';
    };

    return self;
});

}(sofa, document));
